#!/usr/bin/perl -w 

#
# $Id: ibays.pm,v 1.8 2005/09/06 05:49:52 apc Exp $
#

package    esmith::FormMagick::Panel::webhosting;

use strict;

use esmith::FormMagick;
use esmith::AccountsDB;
use esmith::ConfigDB;
use esmith::DomainsDB;
use esmith::cgi;
use esmith::util;
use File::Basename;
use Exporter;
use Carp;
use esmith::php;

our @ISA = qw(esmith::FormMagick Exporter);

our @EXPORT = qw(
    print_ibay_table
    print_ibay_name_field
    print_vhost_message
    max_ibay_name_length
    handle_ibays
    print_save_or_add_button
    wherenext
    validate_up_post
);

our $VERSION = sprintf '%d.%03d', q$Revision: 1.9 $ =~ /: (\d+).(\d+)/;

our $accountdb = esmith::AccountsDB->open();
our $configdb = esmith::ConfigDB->open();

#TODO those 3 variables should be exported from esmith::php
our $defaultdisabledfunc='system,show_source,' .
        'symlink,exec,dl,shell_exec,' .
        'passthru,phpinfo,' .
        'escapeshellarg,escapeshellcmd';
our $BASEPHP = 54;
our %defaultproperties = (
	MemoryLimit 	  => '128M',
	MaxExecutionTime  => '30',
	MaxInputTime	  => '60',
	AllowUrlFopen	  => 'disabled',
	PostMaxSize	  => '20M',
	UploadMaxFilesize => '10M',
	FileUpload	  => 'enabled',
	PHPBaseDir 	  => '/home/e-smith/files/ibays/$key:/var/lib/php/$key',
	DisabledFunctions => $defaultdisabledfunc,
	MailForceSender   => "admin@".$configdb->get_value('DomainName'),
	AllowPHTML 	  => 'disabled',

	ModDav 		  => "disabled",
   	AllowOverride	  => "None",
	FollowSymLinks	  => "disabled",
	Indexes 	  => "enabled",	
	);


# those are ok to be defined here and not in core.
our %sizehash = ('20M' => 'S0020MB', '30M' => 'S0030MB', '40M' => 'S0040MB', '50M' => 'S0050MB',
                         '75M' => 'S0075MB', '100M' => 'S0100MB', '125M' => 'S0125MB', '150M' => 'S0150MB', '175M' => 'S0175MB',
                         '200M' => 'S0200MB', '300M' => 'S0300MB','400M' => 'S0400MB', '500M' => 'S0500MB', '600M' => 'S0600MB',
                         '700M' => 'S0700MB', '800M' => 'S0800MB', '900M' => 'S0900MB', '1000M' => 'S1000MB', '1250M' => 'S1250MB',
                         '1500M' => 'S1500MB', '1750M' => 'S1750MB', '1999M' => 'S2000MB');
our %binary = ('disabled' => 'DISABLED', 'enabled' => 'ENABLED');
our %timehash = ( '60' => 'T001m', '120' => 'T002m', '180' => 'T003m', '240' => 'T004m', '300' => 'T005m',
                         '360' => 'T006m', '420' => 'T007m', '480' => 'T008m', '540' => 'T009m', '600' => 'T010m', '900' => 'T015m',
                         '1800' => 'T030m', '2700' => 'T045m', '3600' => 'T060m', '7200' => 'T120m', '0' => 'UNLIMITED');
our %optionsproperties =(
        MemoryLimit       => {'64M' => 'M0064MB', '128M' => 'M0128MB',
                         '256M' => 'M0256MB', '512M' => 'M0512MB', '768M' => 'M0768MB','1024M' => 'M1024MB' ,'2048M' => 'M2048MB','3072M' => 'M3072MB','4096M' => 'M4096MB','61444M' => 'M6144MB','8192M' => 'M8192MB'},
        MaxExecutionTime  => {%timehash},
        MaxInputTime      => {%timehash},
        AllowUrlFopen     => {%binary},
        PostMaxSize       => {%sizehash},
        UploadMaxFilesize => {%sizehash},
        FileUpload        => {%binary},
        DisabledFunctions => $defaultdisabledfunc,
        AllowPHTML        => {%binary},

        ModDav            => {%binary},
        AllowOverride     => { 	None =>'None' , 
				All => 'All', 
				AuthConfig => 'AuthConfig', 
				FileInfo => 'FileInfo',
				Indexes => 'Indexes',
				Limit => 'Limit',
				'FileInfo Indexes' => 'FileInfo Indexes',
				},
        FollowSymLinks    => {%binary},
        Indexes           => {%binary},
);

=pod 

=head1 NAME

esmith::FormMagick::Panels::ibays - useful panel functions 

=head1 SYNOPSIS

    use esmith::FormMagick::Panels::ibays;

    my $panel = esmith::FormMagick::Panel::ibays->new();
    $panel->display();

=head1 DESCRIPTION

=head2 new();

Exactly as for esmith::FormMagick

=begin testing

$ENV{ESMITH_ACCOUNT_DB} = "10e-smith-base/accounts.conf";
$ENV{ESMITH_CONFIG_DB} = "10e-smith-base/configuration.conf";
$ENV{ESMITH_DOMAINS_DB} = "10e-smith-base/domains.conf";

use_ok('esmith::FormMagick::Panel::ibays');
use vars qw($panel);
ok($panel = esmith::FormMagick::Panel::ibays->new(), 
    "Create panel object");
isa_ok($panel, 'esmith::FormMagick::Panel::ibays');

{ package esmith::FormMagick::Panel::ibays;
  our $accountdb;
  ::isa_ok($accountdb, 'esmith::AccountsDB');
}

=end testing

=cut

sub new 
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = esmith::FormMagick::new($class);
    $self->{calling_package} = (caller)[0];

    return $self;
}

=head1 HTML GENERATION ROUTINES

Routines for generating chunks of HTML needed by the panel.

=head1 ROUTINES FOR FILLING IN FIELDS

=head2 print_ibay_table

Prints out the ibay table on the front page.

=for testing
my $self = esmith::FormMagick::Panel::ibays->new();
$self->{cgi} = CGI->new("");
can_ok('main', 'print_ibay_table');
$self->print_ibay_table();
like($_STDOUT_, qr/NAME/, "Found NAME header in table output");
#like($_STDOUT_, qr/testibay/, "Found test ibay in user table output");
#like($_STDOUT_, qr/ff0000/, "Found red 'reset password' output");

=cut

sub print_ibay_table {
    my $self = shift;
    my $q = $self->{cgi};
    my $name        = $self->localise('NAME');
    my $description = $self->localise('DESCRIPTION');
    my $modify      = $self->localise('MODIFY');
    my $action_h    = $self->localise('ACTION');
    
    my @ibays = $accountdb->ibays();

    unless ( scalar @ibays )
    {
        print $q->Tr($q->td($self->localise('NO_IBAYS')));
        return "";
    }

    print $q->start_table({-CLASS => "sme-border"}),"\n";
    print $q->Tr (
                  esmith::cgi::genSmallCell($q, $name,"header"),
                  esmith::cgi::genSmallCell($q, $description,"header"),
                  esmith::cgi::genSmallCell($q, $action_h,"header", 3)),"\n";
    my $scriptname = basename($0);

    foreach my $i (@ibays) 
    {
        my $ibayname = $i->key();
        my $ibaydesc = $i->prop('Name');

        my $modifiable = $i->prop('Modifiable') || 'yes';


        my $params = $self->build_ibay_cgi_params($ibayname, $i->props());


        my $href = "$scriptname?$params&action=modify&wherenext=";

        my $actionModify = '&nbsp;';
        if ($modifiable eq 'yes')
        {
	    $actionModify .= $q->a({href => "${href}CreateModify"},$modify)
                      . '&nbsp;';
        }

 
        print $q->Tr (
            esmith::cgi::genSmallCell($q, $ibayname,"normal"),
            esmith::cgi::genSmallCell($q, $ibaydesc,"normal"),
            esmith::cgi::genSmallCell($q, $actionModify,"normal"));
    }

    print $q->end_table,"\n";

    return "";
}

sub build_ibay_cgi_params {
    my ($self, $ibayname, %oldprops) = @_;

    #$oldprops{'description'} = $oldprops{Name};
    #delete $oldprops{Name};

    my %props = (
        page    => 0,
        page_stack => "",
        #".id"         => $self->{cgi}->param('.id') || "",
        name => $ibayname,
        #%oldprops
    );

    return $self->props_to_query_string(\%props);
}

*wherenext = \&CGI::FormMagick::wherenext;
sub print_ibay_name_field {
    my $self = shift;
    my $in = $self->{cgi}->param('name') || '';
    my $action = $self->{cgi}->param('action') || '';
    my $maxLength = $configdb->get('maxIbayNameLength')->value;
    print qq(<tr><td colspan="2">) . $self->localise('NAME_FIELD_DESC',
        {maxLength => $maxLength}) . qq(</td></tr>);
    print qq(<tr><td class="sme-noborders-label">) . 
        $self->localise('NAME_LABEL') . qq(</td>\n);
    if ($action eq 'modify' and $in) {
        print qq(
            <td class="sme-noborders-content">$in 
            <input type="hidden" name="name" value="$in">
            <input type="hidden" name="action" value="modify">
            </td>
        );

        # Read the values for each field from the accounts db and store
        # them in the cgi object so our form will have the correct 
        # info displayed.
        my $q = $self->{cgi};
        my $rec = $accountdb->get($in);
        if ($rec)
        {
            $q->param(-name=>'description',-value=>
                $rec->prop('Name')); # yes we keep this one simple
            $q->param(-name=>'indexes',-value=>
                (get_current_value($self,'Indexes')));
            $q->param(-name=>'followSymLinks',-value=>
                (get_current_value($self,'FollowSymLinks'))); 
            $q->param(-name=>'allowOverride',-value=>
                (get_current_value($self,'AllowOverride')));  
            $q->param(-name=>'allowUrlFopen',-value=>
                (get_current_value($self,'AllowUrlFopen'))); 
            $q->param(-name=>'memorylimit',-value=>
                (get_current_value($self,'MemoryLimit'))); 
            $q->param(-name=>'uploadmaxfilesize',-value=>
                (get_current_value($self,'UploadMaxFilesize')));  
            $q->param(-name=>'postmaxsize',-value=>
                (get_current_value($self,'PostMaxSize'))); 
            $q->param(-name=>'maxexecutiontime',-value=>
                (get_current_value($self,'MaxExecutionTime')));
            $q->param(-name=>'maxinputtime',-value=>
                (get_current_value($self,'MaxInputTime')));
            $q->param(-name=>'fileupload',-value=>
                (get_current_value($self,'FileUpload')));
            $q->param(-name=>'allowphtml',-value=>
                (get_current_value($self,'AllowPHTML')));
	    $q->param(-name=>'mailforcesender',-value=>
                ($rec->prop('MailForceSender'))); # yes we keep this one simple
		my $df = ($rec->prop('DisabledFunctions'))? $rec->prop('DisabledFunctions') : $defaultdisabledfunc;
	    $q->param(-name=>'disabledfunctions',-value=>
                ($df)); # yes we want todo this one this way.
            $q->param(-name=>'phpbasedir',-value=>
                ($rec->prop('PHPBaseDir'))); # yes we keep this one simple
            $q->param(-name=>'modDav',-value=>
                (get_current_value($self,'ModDav')));       
	    # we set phpversion using function called by cgi file           
        }
    } else {
        print qq(
            <td><input type="text" name="name" value="$in">
            <input type="hidden" name="action" value="create">
            </td>
        );
    }

    print qq(</tr>\n);
    return undef;

}


=pod

=head2 print_vhost_message()

Prints a warning message that vhosts whose content is this ibay will be
modified to point to primary site.

=for testing
$panel->{cgi} = CGI->new();
$panel->{cgi}->param(-name=>'name', -value=>'bar');
is($panel->print_vhost_message(), undef, 'print_vhost_message');

=cut

sub print_vhost_message {
    my $self = shift;
    my $q = $self->{cgi};
    my $name = $q->param('name');

    my $domaindb = esmith::DomainsDB->open();
    my @domains = $domaindb->get_all_by_prop(Content => $name);
    my $vhostListItems = join "\n",
        (map ($q->li($_->key." ".$_->prop('Description')),
        @domains));
    if ($vhostListItems)
    {
        print $self->localise('VHOST_MESSAGE', {vhostList => $vhostListItems});
    }
    return undef;
}

=head1 THE ROUTINES THAT ACTUALLY DO THE WORK

=for testing
can_ok('main', 'handle_ibays');

=cut

sub handle_ibays {
    my ($self) = @_;
    

    if ($self->cgi->param("action") eq "create") {
        $self->create_ibay();
    } else {
        $self->modify_ibay();
    }
}

=head2 print_save_or_add_button()
=cut

sub print_save_or_add_button {
    my ($self) = @_;

    my $action = $self->cgi->param("action") || '';
    if ($action eq "modify") {
        $self->print_button("SAVE");
    } else {
        $self->print_button("ADD");
    }

}


sub modify_ibay {
    my ($self) = @_;
    my $name = $self->cgi->param('name');
    if (my $acct = $accountdb->get($name)) {
        if ($acct->prop('type') eq 'ibay') {
 
    my %doing=(     Indexes         => 'indexes',
                FollowSymLinks  => 'followSymLinks',   
                AllowOverride   => 'allowOverride',
                ModDav          => 'modDav',
                PHPVersion      => 'PHPVersion',
                AllowUrlFopen   => 'allowUrlFopen', 
                MemoryLimit     => 'memorylimit',    
                FileUpload      => 'fileupload',    
                UploadMaxFilesize   => 'uploadmaxfilesize', 
                PostMaxSize     => 'postmaxsize', 
                MaxExecutionTime     => 'maxexecutiontime',
                MaxInputTime    => 'maxinputtime',
                AllowPHTML      => 'allowphtml',
                MailForceSender => 'mailforcesender',
                DisabledFunctions => 'disabledfunctions',
                PHPBaseDir          => 'phpbasedir',
    );
    foreach my $prop (keys %doing) {
	my $value = $self->cgi->param($doing{$prop});
	$value = "" if ( $value eq "$BASEPHP" && $prop eq 'PHPVersion');
	my $default = 	$defaultproperties{$prop} || "";
	# exceptions to handle
	# DisabledFunctions if equal def delprop
	if ($prop eq 'DisabledFunctions' && $value eq $default) {
		$acct->delete_prop($prop) ;
		next;
	}
	# MailForceSender if empty delprop
	if ($prop eq 'MailForceSender'  && $value eq "" ) {
		$acct->delete_prop($prop) ;
		next
	}
	# PHPBaseDir if empty delprop
	if ($prop eq 'PHPBaseDir' && $value eq "" ) {
                $acct->delete_prop($prop) ;
                next
        }
	# others if == default delprop
	if ($value eq "default") {
		$acct->delete_prop($prop) ;		
		next;
	}
	$acct->merge_props($prop => $value);
	#TODO store them in a hash and call merge_props once
    }

            # Untaint $name before use in system()
            $name =~ /(.+)/; $name = $1;
            if (system ("/sbin/e-smith/signal-event", "webhosting-modify", 
                $name) == 0) 
            {
                $self->success("SUCCESSFULLY_MODIFIED_IBAY");
            } else {
                $self->error("ERROR_WHILE_MODIFYING_IBAY");
            }
        } else {
            $self->error('CANT_FIND_IBAY');
        }
    } else {
        $self->error('CANT_FIND_IBAY');
    }
}


=head2 validate_up_post
 
verify that the upload_max_filesize value is not greater than the post_max_size value. If yes then display an error message.

=cut
sub validate_up_post{
    my $self = shift;
    my $upmaxfilesize       = $self->cgi->param('uploadmaxfilesize');
    my $postmaxsizeform     = $self->cgi->param('postmaxsize');
##set value to "0M" if disabled in order to compare uploadmaxfilesize and postmaxsize
        $upmaxfilesize = "0M" if $upmaxfilesize eq 'disabled';
	$upmaxfilesize = $defaultproperties{'UploadMaxFilesize'} if $upmaxfilesize eq 'default';
        $postmaxsizeform = "0M" if $postmaxsizeform eq 'disabled';
	$postmaxsizeform =$defaultproperties{'PostMaxSize'} if $postmaxsizeform eq 'default';
##remove the 'M' unit
    my $upmaxfilesizechop      = chop($upmaxfilesize);
    my $postmaxsizeformchop    = chop($postmaxsizeform);
##test the condition
        if ( $upmaxfilesize > $postmaxsizeform )
        {
    return $self->localise('UPLOADMAXFILESIZE_IS_GREATER_THAN_POSTMAXSIZE');
        }
    else
        {
            return "OK";
        }
}

=head2 get_current_php_value 

=cut
sub get_current_php_value{
    my ($self) = @_;
    my $name = $self->cgi->param('name');
    my $ibay= $accountdb->get($name);
    return "default" unless defined $ibay->prop('PHPVersion');
    return VersionToUse($accountdb->get($name));
}


=head2 get_current_php_value 

=cut
sub get_php_options{
   my ($self) = @_;
   my $translate    = $self->localise('DEFAULT');
   my %options= listPHPVersionHash();
   $options{"default"}="$translate: ".PHPdefault();
   return \%options;
}

=head2 get_current_value ($self, $property)


=cut
sub get_current_value{
    my ($self, $property) = @_;
    my $name = $self->cgi->param('name');
    my $ibay = $accountdb->get($name);
    my $key = $ibay->key;
    my $default = $defaultproperties{$property} || "";
    $default =~ s/\$key/$key/g if $property eq "PHPBaseDir";
    my $value = ($ibay->prop($property))? $ibay->prop($property)  : "default" ; 
    return $value;

}

sub print_options{
    my ($self, $property) = @_;
    my $translate    = $self->localise('DEFAULT');
    my $name = $self->cgi->param('name');
    my $ibay = $accountdb->get($name);
    my $key = $ibay->key;
    my $default = $defaultproperties{$property} || "";
    $default =~ s/\$key/$key/g if $property eq "PHPBaseDir";
    my %opts=%{$optionsproperties{$property}};
    $default=$self->localise(uc($default)) unless ($property eq 'AllowOverride');
    $opts{"default"} = "$translate: ".$default; 
    return \%opts

}

sub print_disabledfunctions{
    my ($self) = @_;
    my $translate    = $self->localise('DESC_DISABLEDFUNCTIONS');
    my $name = $self->cgi->param('name');
    my $ibay= $accountdb->get($name);
    return "$translate : ". $defaultdisabledfunc ;


}

sub print_phpbasedir{
    my ($self) = @_;
    my $translate    = $self->localise('DESC_PHPBASEDIR');
    my $name = $self->cgi->param('name');
    my $ibay= $accountdb->get($name);
    my $key = $ibay->key;
    my $default = $defaultproperties{'PHPBaseDir'} ||'';
    $default =~ s/\$key/$key/g;
    my $basedir = ($ibay->prop('PHPBaseDir')) ? $ibay->prop('PHPBaseDir') : $defaultproperties{'PHPBaseDir'};
    return "$translate : ". $default ;


}


1;


